#!/bin/bash

TARGET_DEVICE=/dev/mmcblk0

DIR_SRC1=/deploy			# on SD card (when booting from SD card)
DIR_SRC2=/run/media/mmcblk0p2/deploy	# on eMMC
RAM_DIR_TMP=/tmp			# if booting from ramdisk, copy files here before erasing source


SERIAL_CONSOLE=ttyS0
SERIAL_DEVICE=/dev/${SERIAL_CONSOLE}
MY_TERMINAL=`tty`


FILELINES=`wc -l < $0`


function script_complete
{
   if [ "$1" != "0" ]; then
      sleep 5
   fi

   if [ "$BOOTED_RAM_DISK" != "0" ]; then
      shutdown -r 0
   fi

   exit $1
}


function message
{
   echo $*
   if [ "$MY_TERMINAL" != "$SERIAL_DEVICE" ]; then
      echo $* > $SERIAL_DEVICE
   fi
}


function execute
{
   if [ -z "$FIRSTLINENUM" ]; then
      FIRSTLINENUM=${BASH_LINENO[0]}
   fi

   PROGRESS=$(( ( ( ${BASH_LINENO[0]} - $FIRSTLINENUM ) * 100 ) / ( $FILELINES - $FIRSTLINENUM ) ))

   echo "Command (${PROGRESS}%): $*"
   $*
   if [ $? -ne 0 ]; then
      message
      message "ERROR: $BASH_SOURCE:${BASH_LINENO[0]}: executing $*"
      message
      script_complete 1
   fi
}


function check_if_main_drive
{
   BOOT_DEV=`mount | grep " on / type " | awk '{print $1}'`

   if [ $? -ne 0 -o -z "$BOOT_DEV" ]; then
      message "WARNING: not able to determine current filesystem device"
      script_complete 1
   else
      if [ "$BOOT_DEV" == "/dev/ram0" ]; then
         BOOTED_RAM_DISK=1
      else
         BOOTED_RAM_DISK=0
      fi

      message "Boot device is: $BOOT_DEV"
      echo $BOOT_DEV | grep "$TARGET_DEVICE" > /dev/null
      if [ $? -eq 0 ]; then
         message "ERROR: ${TARGET_DEVICE} seems to be current boot device"
         script_complete 1
      fi
   fi
}


function unmount
{
   for dev in $*; do
      echo "unmounting device '$dev'"
      umount $dev
   done
}


function unmount_all
{
   unmount `ls -d ${TARGET_DEVICE}p? ${TMP_DIR}/partition?`
}


function SetSourceVars
{
   SRC_FILE_MLO1=${SRC_DIR_ROOTFS}/MLO
   SRC_FILE_MLO2=${SRC_DIR_ROOTFS}/MLO.byteswap
   SRC_FILE_U_BOOT=${SRC_DIR_ROOTFS}/u-boot.img
   SRC_FILE_ROOTFS_XZ=${SRC_DIR_ROOTFS}/arago-base-tisdk-image-ultrex.tar.xz

   SRC_DIR_RAMDISK=${DIR_SRC/deploy/ramdisk}	# change "deploy" to "ramdisk"
   SRC_FILE_RAMDISK=${SRC_DIR_RAMDISK}/ramdisk.gz
   SRC_FILE_APPLICATION_BZ2=${SRC_DIR_APP}/MegaLiveMotorControl.tar.bz2

   REQUIRED_SRC_FILES="${SRC_FILE_MLO1} ${SRC_FILE_MLO2} ${SRC_FILE_U_BOOT} ${SRC_FILE_ROOTFS_XZ} ${SRC_FILE_APPLICATION_BZ2}"

   DST_DIR_DEPLOY=partition2/deploy
   DST_DIR_RAMDISK=partition2/ramdisk

   SRC_BOOT_DIR=${DIR_SRC/deploy/boot}		# change "deploy" to "boot"
}




check_if_main_drive


# wait until boot is complete (login is ready)
for delay in `seq 120 -1 1`; do
   ps -A | grep agetty > /dev/null
   if [ $? -eq 0 ]; then
      break
   fi
   sleep 1
done


# warn the user before deploy starts and check if deploy should be prevented
if [ "$BOOTED_RAM_DISK" != "0" ]; then
   DELAY=15
else
   DELAY=30
fi

for delay in `seq $DELAY -1 1`; do
   message "Deploy in $delay seconds..."
   if [ -e /home/root/DEPLOY-STOP ]; then
      message "Deploy STOPPED"
      script_complete 1
   fi
   sleep 1
done


if [ ! -b ${TARGET_DEVICE} ]; then
   message "ERROR: ${TARGET_DEVICE} is not a block device file"
   script_complete 1
fi


if [ -e ${DIR_SRC1}/MLO ]; then
   DIR_SRC=${DIR_SRC1}
else
   DIR_SRC=${DIR_SRC2}
fi

if [ ! -d ${DIR_SRC} ]; then
   message "No deploy files available"
   script_complete 1
fi


SRC_DIR_ROOTFS=${DIR_SRC}
SRC_DIR_APP=${DIR_SRC}


SetSourceVars

if [ "$BOOTED_RAM_DISK" != "0" ]; then
   message "Copy deploy files to ${RAM_DIR_TMP}"
   execute cp -vpr ${DIR_SRC} ${RAM_DIR_TMP}

   ##message "Copy ramdisk files to ${RAM_DIR_TMP}"
   ##execute cp -vpr ${SRC_DIR_RAMDISK} ${RAM_DIR_TMP}

   execute mkdir -p ${RAM_DIR_TMP}/boot
   execute cp -vpr ${SRC_BOOT_DIR}/image*.txt ${RAM_DIR_TMP}/boot
   execute cp -vp ${SRC_FILE_APPLICATION_BZ2} ${RAM_DIR_TMP}/deploy

   DIR_SRC=${RAM_DIR_TMP}/deploy

   SetSourceVars
fi


# make sure we have all the files
for NEED in ${REQUIRED_SRC_FILES}; do
   if [ ! -e ${NEED} ]; then
      message "Required file missing: ${NEED}"
      script_complete 1
   fi
done


message "Deploy started"


PARTITION1=${TARGET_DEVICE}p1
PARTITION2=${TARGET_DEVICE}p2
PARTITION3=${TARGET_DEVICE}p3
PARTITION4=${TARGET_DEVICE}p4

# Partion sizes in MB
# 7580M is an 8GB card
PART1_SIZE_MB=64	# boot
PART2_SIZE_MB=1024	# rootfs1
PART3_SIZE_MB=1024	# rootfs2
PART4_SIZE_MB=5468	# application

PART1_SIZE_KB=$((${PART1_SIZE_MB} * 1024))
PART2_SIZE_KB=$((${PART2_SIZE_MB} * 1024))
PART3_SIZE_KB=$((${PART3_SIZE_MB} * 1024))
PART4_SIZE_KB=$((${PART4_SIZE_MB} * 1024))

# Partion sizes in sectors
SECTORS_PER_KB=2
PART1_SIZE_SEC=$((${PART1_SIZE_KB} * ${SECTORS_PER_KB}))
PART2_SIZE_SEC=$((${PART2_SIZE_KB} * ${SECTORS_PER_KB}))
PART3_SIZE_SEC=$((${PART3_SIZE_KB} * ${SECTORS_PER_KB}))

# sector numbers
PART1_FIRST_SEC=1024 # about 1 MB - leave room for u-boot
PART1_LAST_SEC=$((${PART1_FIRST_SEC} + ${PART1_SIZE_SEC}))

PART2_FIRST_SEC=$((${PART1_LAST_SEC} + 1))
PART2_LAST_SEC=$((${PART2_FIRST_SEC} + ${PART2_SIZE_SEC}))

PART3_FIRST_SEC=$((${PART2_LAST_SEC} + 1))
PART3_LAST_SEC=$((${PART3_FIRST_SEC} + ${PART3_SIZE_SEC}))

PART4_FIRST_SEC=$((${PART3_LAST_SEC} + 1))


# make sure existing partitions aren't mounted
unmount_all


# zero the master partition table
execute "dd if=/dev/zero of=${TARGET_DEVICE} bs=1024 count=2048"

# zero the start of the partitions to prevent errors from old data
execute "dd if=/dev/zero of=${TARGET_DEVICE} bs=512 count=64 seek=${PART1_FIRST_SEC}"
execute "dd if=/dev/zero of=${TARGET_DEVICE} bs=512 count=64 seek=${PART2_FIRST_SEC}"
execute "dd if=/dev/zero of=${TARGET_DEVICE} bs=512 count=64 seek=${PART3_FIRST_SEC}"
execute "dd if=/dev/zero of=${TARGET_DEVICE} bs=512 count=64 seek=${PART4_FIRST_SEC}"

sync


TMP_DIR=/tmp/deploy/$$
FDISK_TMP_INPUT=${TMP_DIR}/fdisk-tmp.input
FDISK_TMP_CMD=${TMP_DIR}/fdisk-tmp.cmd

execute rm -rf ${TMP_DIR}
execute mkdir -p ${TMP_DIR}

# create fdisk script file
echo "label: dos" > ${FDISK_TMP_INPUT}
echo "label-id: $(($RANDOM*$RANDOM))" >> ${FDISK_TMP_INPUT}
echo "device: ${TARGET_DEVICE}" >> ${FDISK_TMP_INPUT}
echo "unit: sectors" >> ${FDISK_TMP_INPUT}
echo "" >> ${FDISK_TMP_INPUT}
echo "${PARTITION1} : start=${PART1_FIRST_SEC}, size=${PART1_SIZE_SEC}, type=c, bootable" >> ${FDISK_TMP_INPUT}
echo "${PARTITION2} : start=${PART2_FIRST_SEC}, size=${PART2_SIZE_SEC}, type=83" >> ${FDISK_TMP_INPUT}
echo "${PARTITION3} : start=${PART3_FIRST_SEC}, size=${PART3_SIZE_SEC}, type=83" >> ${FDISK_TMP_INPUT}
echo "${PARTITION4} : start=${PART4_FIRST_SEC}, type=83" >> ${FDISK_TMP_INPUT}


# create command file to read script
echo "I" > ${FDISK_TMP_CMD}
echo "${FDISK_TMP_INPUT}" >> ${FDISK_TMP_CMD}
echo "w" >> ${FDISK_TMP_CMD}

unmount_all

execute fdisk -H 255 -S 63 ${TARGET_DEVICE} < ${FDISK_TMP_CMD}


# make partitions
message "Formatting ${PARTITION1}..."
unmount ${PARTITION1}
execute mkfs.vfat -F 32 -n "boot" ${PARTITION1}

message "Formatting ${PARTITION2}..."
unmount ${PARTITION2}
execute mkfs.ext4 -j -L "rootfs1" ${PARTITION2}

message "Formatting ${PARTITION3}..."
unmount ${PARTITION3}
execute mkfs.ext4 -j -L "rootfs2" ${PARTITION3}

message "Formatting ${PARTITION4}..."
unmount ${PARTITION4}
execute mkfs.ext4 -j -L "application" ${PARTITION4}


MOUNT_PART1=${TMP_DIR}/partition1
MOUNT_PART2=${TMP_DIR}/partition2
MOUNT_PART3=${TMP_DIR}/partition3
MOUNT_PART4=${TMP_DIR}/partition4

message "Preparing for Copy..."
execute "mkdir -p ${MOUNT_PART1} ${MOUNT_PART2} ${MOUNT_PART3} ${MOUNT_PART4}"

execute "mount ${PARTITION1} ${MOUNT_PART1}"
execute "mount ${PARTITION2} ${MOUNT_PART2}"
execute "mount ${PARTITION3} ${MOUNT_PART3}"
execute "mount ${PARTITION4} ${MOUNT_PART4}"


message "Copying filesystem to ${PARTITION2}"
execute "tar -xf ${SRC_FILE_ROOTFS_XZ} -C ${MOUNT_PART2}/"

### copy ramdisk file
##execute mkdir -pv ${DST_DIR_RAMDISK}
##execute cp -vp ${SRC_FILE_RAMDISK} ${DST_DIR_RAMDISK}

# copy information files
execute cp -vp ${SRC_BOOT_DIR}/image*.txt ${MOUNT_PART2}/boot

# remove this script from the deployed system
rm ${MOUNT_PART2}${BASH_SOURCE}

message "Copying application to ${PARTITION4}"
execute "mkdir -p ${MOUNT_PART4}/app"
execute "ln -s /run/media/mmcblk0p4/app /run/media/mmcblk0p2/home/root/app"
execute "tar -xf ${SRC_FILE_APPLICATION_BZ2} -C ${MOUNT_PART4}/app"
execute "chown -R root:root ${MOUNT_PART4}/app"

message "Erasing SPI flash"
execute "flash_erase /dev/mtd0 0 0"
execute "flash_erase /dev/mtd1 0 0"

message "Copying to SPI flash"
execute "flashcp ${SRC_FILE_MLO2}   /dev/mtd0"
execute "flashcp ${SRC_FILE_U_BOOT} /dev/mtd1"

message "Erasing u-boot from emmc"
echo 0 > /sys/block/mmcblk0boot1/force_ro
dd if=/dev/zero of=/dev/mmcblk0boot1 bs=512 count=8192

sync

execute find ${MOUNT_PART1} -exec ls -l {} +

unmount_all

execute "rm -rf ${TMP_DIR}"

message "Deploy completed"

if [ "$BOOTED_RAM_DISK" != "0" ]; then
   script_complete 0
else
   shutdown -h 0
fi
